<?php

error_reporting(E_ALL);

require_once("vars.php");

class McData
{
  var $db;
  var $html;
  var $isParentPage = TRUE;
  var $parentId = FALSE;

  function McData()
  {
    $this->db = new Db;
    $this->html = new McHtml;

    if(isset($_GET["mid"]) && strlen($_GET["mid"])>0 &&is_numeric($_GET["mid"]))
    {
      $this->isParentPage = FALSE;
      $this->parentId = $_GET["mid"];
    }
  }

  function page()
  {
    return $this->html->form($this->isParentPage) . $this->posts();
  }

  function posts()
  {
    $html = "";

    $posts = $this->db->getPosts($this->parentId);

    foreach($posts as $key=>$p)
    {
      $numReplies = ($this->isParentPage === TRUE ?
                     $this->db->numChildren($p["id"]) : NULL);

      $html .= $this->html->post(
          $p["id"],
          $p["name"],
          $p["date"],
          $p["message"],
          $this->isParentPage,
          $numReplies
      );
    }

    return $html;
  }
}


class McHtml
{
  var $html;
  var $dateFormat = "g:m T, l F jS Y";
  
  function McHtml($file=FALSE)
  {
    if($file === FALSE)
      $file = Vars::templateFile();

    $parser = new McTemplateParser($file);
    $this->html = $parser->html;
  }

  function post($mid, $name, $date, $message, $showReplyStuff, $replies=NULL)
  {
    $replyWord = "repl" . ($replies == 1 ? "y" : "ies");

    $replyStuff = ($showReplyStuff!==TRUE) ? "" :
      sprintf($this->html["replystuff"], $replies, $replyWord,
              $_SERVER["PHP_SELF"], $mid);

    $name = trim(strip_tags(TextProc::dbUnescape($name)));

    $message = trim(TextProc::tags(TextProc::clickyurls(
        strip_tags(TextProc::dbUnescape($message)))));

    return sprintf($this->html["post"], $name, date($this->dateFormat, $date),
                   $message, $replyStuff);
  }

  function form($makeParent=FALSE)
  {
    return sprintf($this->html["form"],
                   $makeParent === FALSE ? $_GET["mid"] : 0,
                   $_SERVER["REMOTE_ADDR"]);
  }
}


class Db {

  function Db()
  {
    $this->mysql = new MySQL(Vars::dbhost(),
                             Vars::dbuser(),
                             Vars::dbpass(),
                             Vars::dbname());
    $this->mysql->connect();
  }

  # call with integer or FALSE for parents
  function getPosts($parentId)
  {

    if($parentId !== FALSE) #if we're viewing children
      return array_merge(
        # first post should be the parent
        $this->mysql->assocArray(Queries::getParent($parentId)),
        # and here are the children
        $this->mysql->assocArray(Queries::getChildren($parentId)));

    else # get all parents
      return $this->mysql->assocArray(Queries::getParents());

  }

  function numChildren($parentId)
  {
    return $this->mysql->singleResult(Queries::getNumChildren($parentId));
  }
}


class McTemplateParser
{

  var $html = array();

  function McTemplateParser($file)
  {
    $template = fopen($file, "r");
    $current_element = FALSE;

    while($line = fgets($template, 1024)) {

      if($this->isOpeningTag($line)) {
        $current_element = $this->getOpeningTag($line);
        $this->html[$current_element] = "";
      }

      elseif($this->isClosingTag($line))
        $current_element = FALSE;

      # read lines only while we are inside of a @@foo@@ ... @@/foo@@ block,
      # into $this->html["foo"]
      elseif($current_element !== FALSE)
        $this->html[$current_element] .= $line;
    }

    fclose($template);
  }

  function isOpeningTag($line)
  {
      return (bool)preg_match("#^@@[^/](.*)@@$#", trim($line));
  }
  function isClosingTag($line)
  {
      return (bool)preg_match("#^@@/(.*)@@$#", trim($line));
  }
  function getOpeningTag($line)
  {
      return trim(preg_replace("#^@@(.*)@@$#", '$1', $line));
  }
  function getClosingTag($line)
  {
      return trim(preg_replace("#^@@/(.*)@@$#", '$1', $line));
  }

}


class Queries {

  function getParent($parentId)
  {
    $query = <<<SQL
      SELECT id,date,name,message
      FROM msgs
      WHERE id=%d
      ORDER BY date ASC
      LIMIT 0,1
SQL;
    return sprintf($query, $parentId);
  }

  function getChildren($parentId)
  {
    $query = <<<SQL
      SELECT children.id,children.date,children.name,children.message
      FROM msgs as children
      LEFT JOIN msgs as parents
      ON children.pid=parents.id
      WHERE parents.id=%d
      ORDER BY children.date ASC
SQL;
    return sprintf($query, $parentId);
  }

  function getParents()
  {
    $query = <<<SQL
      SELECT id,date,name,message
      FROM msgs
      WHERE pid=0
      ORDER BY date ASC
SQL;
    return $query;
  }

  function getNumChildren($parentId)
  {
    $query = <<<SQL
      SELECT COUNT(children.id)
      FROM msgs as children
      LEFT JOIN msgs as parents
      ON children.pid=parents.id
      WHERE parents.id=%d
SQL;
    return sprintf($query, $parentId);
  }

  function addPost($pid, $date, $name, $msg)
  {
    $query = <<<SQL
        INSERT INTO msgs (pid, date, name, message)
        VALUES (%s, %s, '%s', '%s')
SQL;
    return sprintf($query, $pid, $date, $name, $msg);
  }
}


class MySQL
{
  var $host;
  var $user;
  var $pass;
  var $dbname;
  var $db;

  function MySQL($host, $user, $pass, $dbname)
  {
    $this->host   = $host;
    $this->user   = $user;
    $this->pass   = $pass;
    $this->dbname = $dbname;
  }

  function err($func)
  {
    return sprintf("MySQL Error #%s in MySQL::%s(): %s", mysql_errno(), $func, mysql_error());
  }

  function connect()
  {
    $this->db = mysql_connect($this->host, $this->user, $this->pass);

    if($this->db === FALSE)
      die($this->err("connect"));

    if(mysql_select_db($this->dbname, $this->db) === FALSE)
      die($this->err("connect"));
  }

  function disconnect()
  {
    if(mysql_close($this->db) === FALSE)
      die($this->err("disconnect"));
  }

  function query($query)
  {
    if(($result = mysql_query($query)) === FALSE)
      die($this->err("query"));

    return $result;
  }

  function singleResult($query)
  {
    return mysql_result($this->query($query),0,0);
  }

  function assocArray($query)
  {
    $table = array();
    $result = $this->query($query);

    while($row = mysql_fetch_assoc($result))
      $table[] = $row;

    return $table;
  }
}


class TextProc {

  function clickyurls($text)
  {
    return preg_replace('#((http(s?)://)|(www\.))(\S+)#i',
                        '<a href="http$3://$4$5">$2$4$5</a>', $text);
  }
  function tags($text)
  {
    return preg_replace("#\[([bi])](.*)\[/([bi])]#i", "<$1>$2<$3>", $text);
  }
  function dbEscape($text)
  {
      return (!get_magic_quotes_gpc() ? addslashes($text) : $text);
  }
  function dbUnescape($text)
  {
      return (get_magic_quotes_gpc() ? stripslashes($text) : $text);
  }
}

?>
